###########################################################################
# Copyright (c) 2011-2014 Unixmedia S.r.l. <info@unixmedia.it>
# Copyright (c) 2011-2014 Franco (nextime) Lanza <franco@unixmedia.it>
#
# Domotika System Controller Daemon "domotikad"  [http://trac.unixmedia.it]
#
# This file is part of domotikad.
#
# domotikad is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import json
from twisted.internet import defer, task, reactor, protocol
from nevow import inevow, rend, tags, loaders, flat, athena, stan, guard
import logging, os, sys
from twisted.web import resource, static, server
import time
from txsockjs.factory import SockJSResource, SockJSFactory
import dmjson as dmj
from penguidom.singleton.ajaxconnections import AJAXCONNREG as CONNREG

log = logging.getLogger( 'Webgui' )

curdir=os.path.abspath(os.path.dirname(sys.argv[0]))




class AjaxProtocol(object):


   def messageReceived(self, message, binary):
      f=getattr(self, 'on_command_'+message.split(":")[0], None)
      if f and callable(f):
         log.debug("Received "+str(message.split(":")[0])+" command")
         f(message)
      else:
         self.messageSend(message, binary)

   def on_command_test(self, data):
      log.debug("TEST AJAX COMMAND "+str(data))
      if str(data) == 'test':
         self.messageSend('test:OK')
      splitted = data.split(':')
      if len(splitted) > 1:
         if splitted[1] == 'broadcast':
            self.factory.core.test()

   def connectionMade(self, who):
      log.debug('AJAX connectionMade called by '+str(who))

   def connectionLost(self, who, reason):
      log.debug('AJAX connectionLost called by '+str(who)+' for reason '+str(reason))




class SockJSServerProtocolWrapper(protocol.Protocol, AjaxProtocol):

   def dataReceived(self, data):
      log.info("SockJS RECEIVED: "+str(data))
      self.messageReceived(data, False)

   def messageSend(self, msg, binary=False):
      self.transport.write(msg)

   def connectionMade(self):
      CONNREG.connections[self] = 1
      AjaxProtocol.connectionMade(self, 'sockjs')

   def connectionLost(self, reason):
      del CONNREG.connections[self]
      AjaxProtocol.connectionLost(self, 'sockjs', reason)



class SockJSServerFactory(protocol.Factory):
   
   protocol = SockJSServerProtocolWrapper

   def buildProtocol(self, addr):
      log.debug("SockJS BuildProtocol for "+str(addr))
      p = self.protocol()
      p.core = self.core
      p.factory = self
      return p


def getSocketJSResource(core):
   options = {
      'websocket': True,
      'cookie_needed': False,
      'heartbeat': 25,
      'timeout': 5,
      'streaming_limit': 128 * 1024,
      'encoding': 'cp1252', # Latin1
      #'sockjs_url': '/resources/js/sockjs-0.3.min.js'
      #'sockjs_url': 'https://d1fxtkz8shb9d2.cloudfront.net/sockjs-0.3.js'
   }
   f = SockJSServerFactory()
   f.core = core
   return SockJSResource(f, options)
   
