###########################################################################
# Copyright (c) 2018- Franco (nextime) Lanza <franco@nexlab.it>
#
# Penguidom System client Daemon "penguidomd"  [https://git.nexlab.net/domotika/Penguidom]
#
# This file is part of penguidom.
#
# penguidom is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from twisted.web import server, resource, static, client
from nexlibs import nexcrypt as dmcrypt
try:
   from twisted.web import http
except ImportError:
   from twisted.protocols import http

import time
from nexlibs.utils.blockingdefer import blockingDeferred
from nexlibs.utils import genutils 
import logging
from zope.interface import implements
from twisted.cred import portal, checkers, credentials
from nevow import inevow, rend, tags, loaders, flat, stan, guard
from nevow import static as nstatic
import rest, bot
import os, sys
from twisted import cred
from twisted.internet import defer
from nevow import appserver
import auth
from nexlibs.utils.genutils import configFile

log = logging.getLogger( 'Webgui' )

curdir=os.path.abspath(os.path.dirname(sys.argv[0]))
LOGINFILE=os.path.normpath("/".join([curdir, 'Web/resources/login.html']))
GUIFILE=os.path.normpath("/".join([curdir, 'Web/resources/page.html']))
GUIPARTSDIR=os.path.normpath("/".join([curdir, 'Web/resources/parts/']))

from common import uni, GzipRequest, StaticFile, codeOk, permissionDenied, RedirectToHome, neededPermission
import ajax


WEB_SYSTEM_PATHS=[
   '/',
   '/__LOGOUT__/',
   '/__logout__/',
   '/__LOGOUT__',
   '/__logout__',
   '/__LOGIN__/',
   '/__login__/',
   '/__LOGIN__',
   '/__login__',
   '/favicon.ico',
]

server.version='PenguidomWeb/1.0'


class BasePage(rend.Page):

   addSlash = True

   child_resources = StaticFile(curdir+'/Web/resources/')
   child_download = StaticFile(curdir+'/Web/download/')

   def renderHTML(self, ctx):
      log.debug("renderHTML in BasePage")
      request = inevow.IRequest(ctx)
      session = inevow.ISession(ctx)
      return rend.Page.renderHTTP(self, ctx)


   def __init__(self, avatarId=None, port=80):
      self.port = port
      log.debug("BasePage initialized by " + str(avatarId))
      super(BasePage, self).__init__(self)
      self.avatarId=avatarId
      self.putChild('favicon.ico', static.File(curdir+'/Web/resources/img/favicon.ico'))
      self.putChild('crossdomain.xml', static.File(curdir+'/Web/resources/xml/crossdomain.xml'))
      self.putChild('sockjs.min.js', static.File(curdir+'/Web/resources/js/sockjs.min.js'))
      self.putChild('manifest.json', static.File(curdir+'/Web/resources/manifest.json'))

   def locateChild(self, ctx, name):
      log.debug("LocateChild in BasePage (%s)" % str(name))
      return rend.Page.locateChild(self, ctx, name)

   def childFactory(self, ctx, name):
      request = inevow.IRequest(ctx)
      log.debug("No child found (%s)" % name)
      return permissionDenied()




class GuiPage(BasePage):

   addSlash = True
   html="""<html><head></head><body><div>Cannot find GuiPage template files</div></body></html>"""
   
   def __init__(self, avatarid):
      self.avatarId = avatarid
      super(GuiPage, self).__init__(self)
      if os.path.isfile(GUIFILE):
         try:
            lf = open(GUIFILE, "r")
            self.html = lf.read()
            lf.close()
         except:
            pass


   def renderHTTP( self, ctx):
      request = inevow.IRequest(ctx)
      return self.getStandardHTML(request.uri)

   def getStandardHTML(self, path):

      log.info(path)
      html = self.html
      html = html.replace("@SCRIPT@", "")
      plist = ''
      phome = ''
      for p in self.core.getPluginList():
         plist+="".join(['<a href="/gui/plugin/', str(p), '">', str(p), '</a>\n']) 
         plug = self.core.getPlugin(p)
         if plug:
            log.debug("Plugin getWebHome for "+str(p)+" "+str(plug))
            phome+=str(plug.getWebHome())
      html = html.replace("@PLUGINS@", plist)
      html = html.replace("@CONTENT@", phome)
      return html

   def childFactory(self, ctx, name):
      log.debug("GuiPage childFactory")
      return self


class RootPage(BasePage):

   logged = False
   childvars = {}
   isLeaf = False
   perms = False
   addSlash = True


   def __init__(self, avatarId=None, port=80):
      self.port = port
      log.debug("Root page initialized by " + str(avatarId))
      super(RootPage, self).__init__(self)

   def child_gui(self, ctx):
      log.debug("Enter GUI page")
      gui = GuiPage(self.avatarId)
      gui.core = self.core
      return gui

   def child_botauth(self, ctx):
      log.debug("BOT AUTH callback received")
      botauth = bot.BotAuth()
      botauth.core = self.core
      return botauth


   def child_rest(self, ctx):
      log.debug("Enter Rest Page")
      self.rest = rest.RestPages()
      self.rest.core = self.core
      return self.rest

   def child_rawplugin(self, ctx):
      log.debug("Trying Plugins Page")
      request = inevow.IRequest(ctx)
      pl=request.path.split("/")
      if len(pl)>2:
         pname=pl[2]
         pconf=os.path.normpath("/".join([curdir, 'plugins', pname, 'conf', pname+".conf" ]))
         log.debug("trying to read "+str(pconf))
         if os.path.isfile(fconf):
            try:
               pcfg=configFile(pconf)
               pcfg.readConfig()
               port=int(pcfg.get('web','port'))
            except:
               port=False
               log.debug("Cannot read config file for plugin "+pname)
            if port:
               self._sendProxySession(request, ctx)
               log.debug("Proxying to plugin path "+str(request.path))
               return proxy.WebProxyResource('localhost', port, path='/', remove=1)
         else:
            log.debug("Plugin hasn't a conf file to read")
      else:
         log.debug("no plugin name in request")
      return self.childFactory(ctx, 'rawplugin')

   def child_(self, ctx):
      log.debug("Gui redirection")
      html = """
         Redirecting...
      """
      request = inevow.IRequest(ctx)

      request.setHeader('Location', 'gui/');
      request.setResponseCode(302)
      return html

   def locateChild(self, ctx, name):
      log.debug("LocateChild in RootPage (%s)" % str(name))
      return rend.Page.locateChild(self, ctx, name)

   def childFactory(self, ctx, name):
      request = inevow.IRequest(ctx)
      log.debug("RootPage No child found (%s)" % str(name))
      return permissionDenied()


### Authentication
from twisted.cred.error import UnauthorizedLogin
from nevow import  url

class SessionWrapper(guard.SessionWrapper):

   def checkLogin(self, ctx, session, segments, *a, **kw):
      try:
         session.saveargs = session.args
      except:
         pass
      request = inevow.IRequest(ctx)
      return guard.SessionWrapper.checkLogin(self, ctx, session, segments, *a, **kw)

   def incorrectLoginError(self, error, ctx, segments, loginFailure):
      """ Used as an errback upon failed login, returns a 2-tuple of a failure URL
      with the query argument 'login-failure' set to the parameter
      loginFailure, and an empty list of segments, to redirect to that URL.
      The basis for this error URL, i.e. the part before the query string, is
      taken either from the 'referer' header from the given request if one
      exists, or a computed URL that points at the same page that the user is
      currently looking at to attempt login.  Any existing query string will
      be stripped.
      """
      request = inevow.IRequest(ctx)
      session = inevow.ISession(ctx)
      error.trap(UnauthorizedLogin)
      referer = request.getHeader("referer")
      if 'botauth' in segments and len(segments) is 3:
         segments = ('bot', segments[1], 'loginfailed', '')
         u = guard.urlToChild(ctx, *segments)
         for k in session.saveargs.keys():
            if k not in ['username', 'password'] and session.saveargs[k][0] not in ['Login']:
               u = u.add(k, session.saveargs[k][0])
      else:
         if referer is not None:
            u = url.URL.fromString(referer)
         else:
            u = guard.urlToChild(ctx, *segments)
         u = u.clear()
         u = u.add('login-failure', loginFailure)

      return u, ()


   def renderHTTP( self, ctx):
      request = inevow.IRequest(ctx)
      host=request.getHeader('host')
      log.info("USERNAME: "+str(request.getUser()))
      log.debug("SessionWrapper HOST CALLED: "+str(host))
      return guard.SessionWrapper.renderHTTP(self, ctx)


   def locateChild(self, ctx, segments):
      request = inevow.IRequest(ctx)
      name = "/".join(segments)
      if name=='':
         name="/"
      log.info("SessionWrapper locateChild "+str(name)+" from IP:"+str(request.getClientIP()))
      if name and name.startswith('bot/'):
         # Bypass for chat bots
         chatbot = bot.BotPages()
         chatbot.core = self.core
         return (chatbot,  segments[1:])

      host=request.getHeader('host')
      log.debug("SessionWrapper HOST CALLED: "+str(host))
      if request.getUser() and request.getPassword():
         log.info("BASIC AUTH REQUESTED FOR USER "+str(request.getUser()))
         request.args["username"] = [request.getUser()]
         request.args["password"] = [request.getPassword()]

      return guard.SessionWrapper.locateChild(self, ctx, segments)
      

def noLogout():
   return None


class LogoutPage(rend.Page):

   addSlash = True

   def __init__(self, locpath):
      self.locpath=locpath.replace("/__LOGOUT__/","/__logout__/")

   def renderHTTP(self, ctx):
      request = inevow.IRequest(ctx)
      rmec=request.getCookie("Penguidom_rme")
      # XXX Come si fa a capire SE esiste? ritorna davvero None?
      if rmec:
         request.addCookie("Penguidom_rme", str(rmec), path="/", secure=True,
            expires=http.datetimeToString(time.time()))
      request.setHeader('Location', self.locpath)
      request.setResponseCode(302)
      log.debug("SPECIAL LOGOUT "+self.locpath)
      return 'Logging out...'

   def childFactory(self, ctx, data):
      return self



class RootAuthPage(RootPage):

   def __init__(self, avatarId=None, port=80, mind=None):
      super(RootAuthPage, self).__init__(self)
      self.logged=True
      self.mind = mind
      self.perms = avatarId[0]

   def logout(self):
      log.debug("LOGOUT CALLED")
      self.logged=False


   def locateChild(self, ctx, name):
      request = inevow.IRequest(ctx)
      if not self.mind.loginsaved:
         log.debug("LocateChild in RootAuthPage")
         self.mind.loginsaved = True
         self.mind.perms = self.perms
         if 'rememberMe' in self.mind.args:
            log.debug("Setting rememberMe cookie for avatar "+str(self.avatarId))
            rme=dmcrypt.B64AESEncrypt(str(self.core.configGet('web', 'cookie_aeskey')), self.perms.salt, 
                                       " ".join([self.perms.username, self.perms.hash, self.perms.salt]))
            try:
               expire=http.datetimeToString(time.time() + 3600*24*365*50)
            except:
               expire=http.datetimeToString(time.time() + 3600*24*365*10)

            request.addCookie("Penguidom_rme", str(self.perms.username)+':'+rme, 
               path="/", secure=True, expires=expire)
      return RootPage.locateChild(self, ctx, name)


   def childFactory(self, ctx, name):
      log.debug("RootAuth childFactory")
      request = inevow.IRequest(ctx)
      if request.path.startswith("/__LOGOUT__/"):
         log.debug("__LOGOUT__ CALLED")
         return LogoutPage(request.path)
      return RootPage.childFactory(self, ctx, name)



class LoginPage(rend.Page):

   addSlash = True

   html="""<html><head>
<title>Penguidom GUI</title>
@SCRIPT@
</head>
<body class="@THEME@">
<form method="post" name="loginform" action="@PATH@">
<fieldset id="form"><legend>Login</legend>
<p>Username: <input type="text" name="username" size="20" value="@USERNAME@" /></p>
<p>Password: <input type="password" name="password" size="20" value="@PASSWORD@" /></p>
<p>Remember me: <input type="checkbox" name="rememberMe" value="Yes" @CHECKED@ /></p>
<p align="center"><input type="submit" value="Login" /></p>
</fieldset>
</form>
</body></html>
   """

   child_img = StaticFile(curdir+'/Web/resources/img/')

   def __init__(self, *a, **kw):
      self.putChild('favicon.ico', static.File(curdir+'/Web/resources/img/favicon.ico'))
      self.putChild('manifest.json', static.File(curdir+'/Web/resources/manifest.json'))
      if os.path.isfile(LOGINFILE):
         try:
            lf = open(LOGINFILE, "r")
            self.html = lf.read()
            lf.close()
         except:
            pass

      rend.Page.__init__(self, *a, **kw)

   def renderHTTP( self, ctx):
      rme=False

      request = inevow.IRequest(ctx)
      host=request.getHeader('host')
      log.info("LOGIN HOST CALLED: "+str(host))
      cookies=request.getHeader('cookie')
      if cookies:
         cookies=cookies.replace(" ","").split(';')
         for cookie in cookies:
            cookiename = cookie.split("=")[0]
            if cookiename.startswith('Penguidom_session'):
               log.info("REMOVE COOKIE: "+str(request.getCookie(cookie.split("=")[0])))
               # XXX This won't work as expected if user is logging in with path != from "/"
               # Also, is cookie secure even for http requests?
               request.addCookie(cookiename, cookie.split("=")[1],  expires=http.datetimeToString(time.time()), path="/", secure=True)
            elif cookiename.startswith('Penguidom_rme'):
               rmec=str(request.getCookie("Penguidom_rme"))
               log.info("RememberMe COOKIE FOUND: "+rmec)
               rmecl = rmec.split(':')
               if len(rmecl) > 1:
                  user = str(rmecl[0])
                  rme = auth.getUserLine(user)

         
      if request.path.startswith("/rest/") and '/keepalive' in request.path:
         request.setHeader('content-type', 'application/json')
         return '{"data": "SLOGGEDOUT", "uri": "'+request.path+'", "ts": '+str(time.time())+', "result": "succeed"}'

      if not rme:
         log.info("LOGIN FORM FOR PATH "+request.uri)
         return self.getStandardHTML(request.uri)
      else:
         log.info("LOGIN FROM COOKIE FOR PATH "+request.uri)
         return self.rmelogin(rme, request, rmec)


   def getStandardHTML(self, path):
      log.info(path)
      html = self.html.replace("@PATH@", '/__login__'+path)
      html = html.replace("@USERNAME@", '')
      html = html.replace("@PASSWORD@", '')
      html = html.replace("@SCRIPT@", "")
      html = html.replace("@CHECKED@", "")
      return html

   def getScript(self, path):
      return '<script> window.onload=function(){ window.setTimeout(function() {document.loginform.submit();}, 200); };</script>'

   def rmelogin(self, userq, req, has):

      rme=dmcrypt.B64AESDecrypt(str(self.core.configGet('web', 'cookie_aeskey')), userq['salt'], has.split(":", 1)[1])
      if len(rme.split()) == 3:
         u, h, s = rme.split()
         if userq['user'] == u and userq['hash'] == h:
            log.info("Cookie login succeed for user "+userq['user'])
            try:
               expire=http.datetimeToString(time.time() + 3600*24*365*50)
            except:
               expire=http.datetimeToString(time.time() + 3600*24*365*10)

            req.addCookie("Penguidom_rme", has,
                  path="/", secure=True, expires=expire)

            html = self.html.replace("@PATH@", '/__login__'+req.uri)
            html = html.replace("@USERNAME@", str(userq['user']))
            html = html.replace("@PASSWORD@", str(userq['hash']))
            html = html.replace("@CHECKED@", "checked")
            html = html.replace("@SCRIPT@", str(self.getScript(req.path)))
            log.debug("login html")
            return html

      req.addCookie("Penguidom_rme", has, path="/", secure=True, expires=http.datetimeToString(time.time()))
      return self.getStandardHTML(req.path)

   def child_bot(self, ctx):
      log.debug("BOT callback received")
      chatbot = bot.BotPages()
      chatbot.core = self.core
      return chatbot

   def childFactory(self, ctx, name):
      log.debug("Login childFactory")
      return self


class PenguidomAuthRealm(object):
   """A simple implementor of cred's IRealm.
      For web, this gives us the LoggedIn page.
   """
   implements(portal.IRealm)

   def __init__(self, port):
      self.port = port

   def requestAvatar(self, authres, mind, *interfaces):
      avatarId = authres

      log.debug("Avatar is: "+str(avatarId)+' and mind + interfaces '+str(mind)+' '+str(interfaces))
      for iface in interfaces:
         if iface is inevow.IResource:
            # do web stuff
            if avatarId is checkers.ANONYMOUS:
               resc = LoginPage()
               resc.realm = self
               resc.core = self.core
               return (inevow.IResource, resc, noLogout)
            else:
               # Here it is, we are authenticated.
               # avatarId should contain the results of the authentication
               # process
               resc = RootAuthPage(avatarId, self.port, mind)
               resc.realm = self
               resc.core = self.core
               resc.putChild('sockjs', ajax.getSocketJSResource(self.core))
               return (inevow.IResource, resc, resc.logout)

      raise NotImplementedError("Can't support that interface.")


def getAuthResource(core):
   realm = PenguidomAuthRealm(core.configGet('web', 'sslport'))
   realm.core = core
   porta = portal.Portal(realm)
   mycheck=auth.clientAuth()
   mycheck.core = core
   porta.registerChecker(checkers.AllowAnonymousAccess(), credentials.IAnonymous)
   porta.registerChecker(mycheck)
   site = SessionWrapper(porta, 'Penguidom_session', mindFactory=auth.mindFactory)
   site.core = core
   return site


class PageParts(object):

   parts = {}

   def _getPartFile(self, part):
      if part in self.parts.keys():
         return self.parts[part]
      
      partfile = ''.join([GUIPARTSDIR, '/', part, '.html'])
      log.info(partfile)
      if os.path.isfile(partfile):
         try:
            lf = open(partfile, "r")
            html = lf.read()
            lf.close()
            self.parts[part] = html
            return html
         except:
            pass
      return ''


   def getColumn(self, title="Title", content=""):
      col = self._getPartFile('column')
      col = col.replace('@COLTITLE@', title)
      col = col.replace('@COLCONTENT@', content)
      return col

   def getButton(self, title="Button", act="Act"):
      btn = self._getPartFile('button')
      btn = btn.replace('@BTNNAME@', title)
      btn = btn.replace('@BTNACT@', act)
      return btn

   def getOnOffSwitch(self, sid='myonoffswitch', on='ON', off='OFF'):
      swh = self._getPartFile('onoffswitch')
      swh = swh.replace('@ON@', on)
      swh = swh.replace('@OFF@', off)
      swh = swh.replace('@ID@' , sid)
      return swh
